/* This file is automatically generated.  Do not edit. */
   S_(SECCLASS_DIR, file, 0x00020000UL)
   S_(SECCLASS_FILE, file, 0x00020000UL)
   S_(SECCLASS_LNK_FILE, file, 0x00020000UL)
   S_(SECCLASS_CHR_FILE, file, 0x00020000UL)
   S_(SECCLASS_BLK_FILE, file, 0x00020000UL)
   S_(SECCLASS_SOCK_FILE, file, 0x00020000UL)
   S_(SECCLASS_FIFO_FILE, file, 0x00020000UL)
   S_(SECCLASS_SOCKET, socket, 0x00400000UL)
   S_(SECCLASS_TCP_SOCKET, socket, 0x00400000UL)
   S_(SECCLASS_UDP_SOCKET, socket, 0x00400000UL)
   S_(SECCLASS_RAWIP_SOCKET, socket, 0x00400000UL)
   S_(SECCLASS_NETLINK_SOCKET, socket, 0x00400000UL)
   S_(SECCLASS_PACKET_SOCKET, socket, 0x00400000UL)
   S_(SECCLASS_KEY_SOCKET, socket, 0x00400000UL)
   S_(SECCLASS_UNIX_STREAM_SOCKET, socket, 0x00400000UL)
   S_(SECCLASS_UNIX_DGRAM_SOCKET, socket, 0x00400000UL)
   S_(SECCLASS_IPC, ipc, 0x00000200UL)
   S_(SECCLASS_SEM, ipc, 0x00000200UL)
   S_(SECCLASS_MSGQ, ipc, 0x00000200UL)
   S_(SECCLASS_SHM, ipc, 0x00000200UL)
   S_(SECCLASS_NETLINK_ROUTE_SOCKET, socket, 0x00400000UL)
   S_(SECCLASS_NETLINK_FIREWALL_SOCKET, socket, 0x00400000UL)
   S_(SECCLASS_NETLINK_TCPDIAG_SOCKET, socket, 0x00400000UL)
   S_(SECCLASS_NETLINK_NFLOG_SOCKET, socket, 0x00400000UL)
   S_(SECCLASS_NETLINK_XFRM_SOCKET, socket, 0x00400000UL)
   S_(SECCLASS_NETLINK_SELINUX_SOCKET, socket, 0x00400000UL)
   S_(SECCLASS_NETLINK_AUDIT_SOCKET, socket, 0x00400000UL)
   S_(SECCLASS_NETLINK_IP6FW_SOCKET, socket, 0x00400000UL)
   S_(SECCLASS_NETLINK_DNRT_SOCKET, socket, 0x00400000UL)
   S_(SECCLASS_NETLINK_KOBJECT_UEVENT_SOCKET, socket, 0x00400000UL)
   S_(SECCLASS_APPLETALK_SOCKET, socket, 0x00400000UL)
   S_(SECCLASS_DCCP_SOCKET, socket, 0x00400000UL)
   S_(SECCLASS_DB_DATABASE, database, 0x00000040UL)
   S_(SECCLASS_DB_TABLE, database, 0x00000040UL)
   S_(SECCLASS_DB_PROCEDURE, database, 0x00000040UL)
   S_(SECCLASS_DB_COLUMN, database, 0x00000040UL)
   S_(SECCLASS_DB_BLOB, database, 0x00000040UL)
