/* File name suffixes.  */
S_(BINPOLICY, "/policy/policy")
    S_(CONTEXTS_DIR, "/contexts")
    S_(FILE_CONTEXTS, "/contexts/files/file_contexts")
    S_(HOMEDIR_CONTEXTS, "/contexts/files/homedir_template")
    S_(DEFAULT_CONTEXTS, "/contexts/default_contexts")
    S_(USER_CONTEXTS, "/contexts/users/")
    S_(FAILSAFE_CONTEXT, "/contexts/failsafe_context")
    S_(DEFAULT_TYPE, "/contexts/default_type")
    S_(SECURETTY_TYPES, "/contexts/securetty_types")
    S_(BOOLEANS, "/booleans")
    S_(MEDIA_CONTEXTS, "/contexts/files/media")
    S_(REMOVABLE_CONTEXT, "/contexts/removable_context")
    S_(CUSTOMIZABLE_TYPES, "/contexts/customizable_types")
    S_(USERS_DIR, "/users/")
    S_(SEUSERS, "/seusers")
    S_(TRANSLATIONS, "/setrans.conf")
    S_(NETFILTER_CONTEXTS, "/contexts/netfilter_contexts")
    S_(FILE_CONTEXTS_HOMEDIR, "/contexts/files/file_contexts.homedirs")
    S_(FILE_CONTEXTS_LOCAL, "/contexts/files/file_contexts.local")
    S_(X_CONTEXTS, "/contexts/x_contexts")
    S_(COLORS, "/secolor.conf")
    S_(VIRTUAL_DOMAIN, "/contexts/virtual_domain_context")
    S_(VIRTUAL_IMAGE, "/contexts/virtual_image_context")
    S_(LXC_CONTEXTS, "/contexts/lxc_contexts")
    S_(FILE_CONTEXT_SUBS, "/contexts/files/file_contexts.subs")
    S_(FILE_CONTEXT_SUBS_DIST, "/contexts/files/file_contexts.subs_dist")
    S_(SEPGSQL_CONTEXTS, "/contexts/sepgsql_contexts")
    S_(BOOLEAN_SUBS, "/booleans.subs_dist")
