/* This file is automatically generated.  Do not edit. */
TB_(common_file_perm_to_string)
    S_("ioctl")
    S_("read")
    S_("write")
    S_("create")
    S_("getattr")
    S_("setattr")
    S_("lock")
    S_("relabelfrom")
    S_("relabelto")
    S_("append")
    S_("unlink")
    S_("link")
    S_("rename")
    S_("execute")
    S_("swapon")
    S_("quotaon")
    S_("mounton")
TE_(common_file_perm_to_string)

TB_(common_socket_perm_to_string)
    S_("ioctl")
    S_("read")
    S_("write")
    S_("create")
    S_("getattr")
    S_("setattr")
    S_("lock")
    S_("relabelfrom")
    S_("relabelto")
    S_("append")
    S_("bind")
    S_("connect")
    S_("listen")
    S_("accept")
    S_("getopt")
    S_("setopt")
    S_("shutdown")
    S_("recvfrom")
    S_("sendto")
    S_("recv_msg")
    S_("send_msg")
    S_("name_bind")
TE_(common_socket_perm_to_string)

TB_(common_ipc_perm_to_string)
    S_("create")
    S_("destroy")
    S_("getattr")
    S_("setattr")
    S_("read")
    S_("write")
    S_("associate")
    S_("unix_read")
    S_("unix_write")
TE_(common_ipc_perm_to_string)

TB_(common_database_perm_to_string)
    S_("create")
    S_("drop")
    S_("getattr")
    S_("setattr")
    S_("relabelfrom")
    S_("relabelto")
TE_(common_database_perm_to_string)

