/* This file is automatically generated.  Do not edit. */
/*
 * Security object class definitions
 */
    S_("null")
    S_("security")
    S_("process")
    S_("system")
    S_("capability")
    S_("filesystem")
    S_("file")
    S_("dir")
    S_("fd")
    S_("lnk_file")
    S_("chr_file")
    S_("blk_file")
    S_("sock_file")
    S_("fifo_file")
    S_("socket")
    S_("tcp_socket")
    S_("udp_socket")
    S_("rawip_socket")
    S_("node")
    S_("netif")
    S_("netlink_socket")
    S_("packet_socket")
    S_("key_socket")
    S_("unix_stream_socket")
    S_("unix_dgram_socket")
    S_("sem")
    S_("msg")
    S_("msgq")
    S_("shm")
    S_("ipc")
    S_("passwd")
    S_("x_drawable")
    S_("x_screen")
    S_("x_gc")
    S_("x_font")
    S_("x_colormap")
    S_("x_property")
    S_("x_selection")
    S_("x_cursor")
    S_("x_client")
    S_("x_device")
    S_("x_server")
    S_("x_extension")
    S_("netlink_route_socket")
    S_("netlink_firewall_socket")
    S_("netlink_tcpdiag_socket")
    S_("netlink_nflog_socket")
    S_("netlink_xfrm_socket")
    S_("netlink_selinux_socket")
    S_("netlink_audit_socket")
    S_("netlink_ip6fw_socket")
    S_("netlink_dnrt_socket")
    S_("dbus")
    S_("nscd")
    S_("association")
    S_("netlink_kobject_uevent_socket")
    S_("appletalk_socket")
    S_("packet")
    S_("key")
    S_("context")
    S_("dccp_socket")
    S_("memprotect")
    S_("db_database")
    S_("db_table")
    S_("db_procedure")
    S_("db_column")
    S_("db_tuple")
    S_("db_blob")
    S_("peer")
    S_("capability2")
    S_("x_resource")
    S_("x_event")
    S_("x_synthetic_event")
    S_("x_application_data")
